use sea_orm::entity::prelude::*;
use serde::{Deserialize, Serialize};
use crate::api::cluster::volume::VolumeParams;
use crate::api::entity::nodes;
use crate::api::entity::nodes::P5xError;
use crate::api::services::Services;


/** Given the API volume params, look up the Volume model instance. */
pub async fn resolve(
    svc: &Services<'_>,
    params: &VolumeParams,
) -> Result<Model, P5xError> {
    let mut q = Entity::find();

    match params.id {
        Some(id) => q = q.filter(Column::VolumeId.eq(id)),
        None => q = q.filter(Column::Name.eq(&params.name)),
    };

    q.one(svc.db)
        .await
        .map_err(P5xError::DbErr)?
        .ok_or(P5xError::BadPrecondition("Could not resolve volume from params: query matched no results"))
}

#[derive(Clone, Debug, PartialEq, Eq, DeriveEntityModel, Deserialize, Serialize, FromForm)]
#[sea_orm(table_name = "volumes")]
pub struct Model {
    #[sea_orm(primary_key)]
    pub volume_id: i32,
    pub name: String,
    pub size_in_bytes: i64,
    pub pve_node_id: i32,
    pub mountpoint_identifier: Option<String>,
    pub mountpoint_host: Option<String>,
    pub mountpoint: Option<String>,
    pub disk_name: Option<String>,
}

impl Model {
    /** Get the Node model for the LXC container this volume is attached to. */
    pub async fn node(&self, svc: &Services<'_>) -> Result<nodes::Model, P5xError> {
        nodes::Entity::find()
            .filter(nodes::Column::PveId.eq(self.pve_node_id))
            .one(svc.db)
            .await
            .map_err(P5xError::DbErr)?
            .ok_or(P5xError::BadPrecondition("Could not find node for volume: pve_node_id does not match any nodes"))
    }

    /** Get the name of this volume as it would appear in the LXC config file. */
    pub async fn qualified_name(&self, svc: &Services<'_>) -> Result<String, P5xError> {
        let node = self.node(svc).await?;
        let storage = &svc.config.pve_storage_pool;
        let storage_type = &svc.config.pve_storage_driver;

        let disk_name = format!("pve-{}.raw", self.name);
        let disk_name = self.disk_name.as_ref().unwrap_or(&disk_name);
        let qn = match storage_type.as_str() {
            "lvm" => format!("{storage}:{disk_name}"),
            _ => format!("{storage}:{}/{disk_name}", node.pve_id),
        };

        Ok(qn)
    }
}

impl Into<VolumeParams> for Model {
    fn into(self) -> VolumeParams {
        VolumeParams {
            id: Some(self.volume_id),
            name: self.name,
            size_in_bytes: self.size_in_bytes,
            mountpoint: self.mountpoint,
        }
    }
}

#[derive(Copy, Clone, Debug, EnumIter, DeriveRelation)]
pub enum Relation {}

impl ActiveModelBehavior for ActiveModel {}
