use std::{env, fs};
use std::fmt::Debug;
use rocket::http;
use rocket::response::status;
use log::error;
use rocket::serde::Deserialize;


/** Global config for the P5x application. */
#[derive(Deserialize)]
pub struct P5xConfig {
    pub pve_host_name: String,
    pub pve_api_host: String,
    pub pve_root_password: String,
    pub pve_storage_pool: String,
    pub pve_storage_driver: String,
    pub k8s_root_password: String,
    pub ssh_pubkey: String,
    pub ssh_privkey: String,
}


/** Read the P5xConfig instance from the corresponding environment variables. */
pub fn read_p5x_config() -> P5xConfig {
    let pubkey_path = env::var("P5X_SSH_PUBKEY_PATH").expect("Missing env: P5X_SSH_PUBKEY_PATH");
    let privkey_path = env::var("P5X_SSH_PRIVKEY_PATH").expect("Missing env: P5X_SSH_PRIVKEY_PATH");

    let config = P5xConfig {
        pve_host_name: env::var("P5X_NODE_HOSTNAME").expect("Missing env: P5X_NODE_HOSTNAME"),
        pve_api_host: env::var("P5X_API_HOST").expect("Missing env: P5X_API_HOST"),
        pve_root_password: env::var("P5X_API_ROOT_PASSWORD").expect("Missing env: P5X_API_ROOT_PASSWORD"),
        pve_storage_pool: env::var("P5X_STORAGE_POOL").expect("Missing env: P5X_STORAGE_POOL"),
        pve_storage_driver: env::var("P5X_STORAGE_DRIVER").expect("Missing env: P5X_STORAGE_DRIVER"),
        k8s_root_password: env::var("P5X_K8S_ROOT_PASSWORD").expect("Missing env: P5X_K8S_ROOT_PASSWORD"),
        ssh_pubkey: fs::read_to_string(&pubkey_path).expect(&format!("Could not read SSH pubkey from file: {pubkey_path}")),
        ssh_privkey: fs::read_to_string(&privkey_path).expect(&format!("Could not read SSH privkey from file: {privkey_path}")),
    };

    config
}


/** Catch-all Rocket helper to generate an HTTP 500 response for the given error. */
pub fn raise_500(e: impl Debug) -> status::Custom<String> {
    error!("Unhandled error: {e:?}");
    status::Custom(http::Status::InternalServerError, format!("An unexpected error has occurred: {e:?}"))
}

